@@ -419,4 +419,7 @@ SERF_DECLARE_DATA const serf_bucket_type_t serf_bucket_type_headers = {
     serf_default_read_bucket,
     serf_headers_peek,
     serf_headers_destroy_and_data,
+    serf_default_snapshot,
+    serf_default_restore_snapshot,
+    serf_default_is_snapshot_set,
 };
