@@ -38,6 +38,9 @@
 #include <apr_network_io.h>
 #include <apr_portable.h>
 #include <apr_strings.h>
+#include <apr_base64.h>
+#include <apr_version.h>
+#include <apr_atomic.h>
 
 #include "serf.h"
 #include "serf_bucket_util.h"
@@ -47,6 +50,14 @@
 #include <openssl/err.h>
 #include <openssl/pkcs12.h>
 
+#ifndef APR_VERSION_AT_LEAST /* Introduced in APR 1.3.0 */
+#define APR_VERSION_AT_LEAST(major,minor,patch)                           \
+    (((major) < APR_MAJOR_VERSION)                                        \
+      || ((major) == APR_MAJOR_VERSION && (minor) < APR_MINOR_VERSION)    \
+      || ((major) == APR_MAJOR_VERSION && (minor) == APR_MINOR_VERSION && \
+               (patch) <= APR_PATCH_VERSION))
+#endif /* APR_VERSION_AT_LEAST */
+
 /*#define SSL_VERBOSE*/
 
 /*
@@ -173,6 +184,7 @@ typedef struct {
 
 struct serf_ssl_certificate_t {
     X509 *ssl_cert;
+    int depth;
 };
 
 /* Returns the amount read. */
@@ -402,6 +414,7 @@ validate_server_certificate(int cert_valid, X509_STORE_CTX *store_ctx)
 
         cert = apr_palloc(subpool, sizeof(serf_ssl_certificate_t));
         cert->ssl_cert = server_cert;
+        cert->depth = depth;
 
         /* Callback for further verification. */
         status = ctx->server_cert_callback(ctx->server_cert_userdata,
@@ -672,11 +685,18 @@ static apr_status_t cleanup_ssl(void *data)
 
 #endif
 
-static int have_init_ssl = 0;
+static apr_uint32_t have_init_ssl = 0;
 
 static void init_ssl_libraries(void)
 {
-    if (!have_init_ssl) {
+    apr_uint32_t val;
+#if APR_VERSION_AT_LEAST(1,0,0)
+    val = apr_atomic_xchg32(&have_init_ssl, 1);
+#else
+    val = apr_atomic_cas(&have_init_ssl, 1, 0);
+#endif
+
+    if (!val) {
 #if APR_HAS_THREADS
         int i, numlocks;
 #endif
@@ -708,8 +728,6 @@ static void init_ssl_libraries(void)
 
         apr_pool_cleanup_register(ssl_pool, NULL, cleanup_ssl, cleanup_ssl);
 #endif
-
-        have_init_ssl = 1;
     }
 }
 
@@ -1150,6 +1168,12 @@ convert_X509_NAME_to_table(X509_NAME *org, apr_pool_t *pool)
     return tgt;
 }
 
+SERF_DECLARE(int)
+serf_ssl_cert_depth(const serf_ssl_certificate_t *cert)
+{
+    return cert->depth;
+}
+
 SERF_DECLARE(apr_hash_t *)
 serf_ssl_cert_issuer(const serf_ssl_certificate_t *cert, apr_pool_t *pool)
 {
@@ -1225,6 +1249,32 @@ serf_ssl_cert_certificate(const serf_ssl_certificate_t *cert, apr_pool_t *pool)
     return tgt;
 }
 
+SERF_DECLARE(const char *)
+serf_ssl_cert_export(const serf_ssl_certificate_t *cert, apr_pool_t *pool)
+{
+    char *binary_cert, *p;
+    char *encoded_cert;
+    int len;
+
+    /* find the length of the DER encoding. */
+    len = i2d_X509(cert->ssl_cert, NULL);
+    if (len < 0) {
+        return NULL;
+    }
+
+    binary_cert = apr_palloc(pool, len);
+    p = binary_cert;
+    len = i2d_X509(cert->ssl_cert, &p); /* p is incremented */
+    if (len < 0) {
+        return NULL;
+    }
+
+    encoded_cert = apr_palloc(pool, apr_base64_encode_len(len));
+    apr_base64_encode(encoded_cert, binary_cert, len);
+    
+    return encoded_cert;
+}
+
 static void serf_ssl_destroy_and_data(serf_bucket_t *bucket)
 {
     ssl_context_t *ctx = bucket->data;
@@ -1330,4 +1380,7 @@ SERF_DECLARE_DATA const serf_bucket_type_t serf_bucket_type_ssl_decrypt = {
     serf_default_read_bucket,
     serf_ssl_peek,
     serf_ssl_decrypt_destroy_and_data,
+    serf_default_snapshot,
+    serf_default_restore_snapshot,
+    serf_default_is_snapshot_set,
 };
